/*!
 * Control program for VME-GbE2
 *
 * Copyright (C) 2016
 *		Bee Beans Technologies Co.,Ltd.
 *		All rights reserved.
 */
#include "vme-gbe-config.h"
#include "vme-gbe-comm.h"
#include <stdio.h>
#include <stdlib.h>
#include <fstream>

int main(int argc, char* argv[])
{
	int n = 2;
	if (argc < n) {
		puts("\nControl program for VME-GbE2");
		printf("Usage: %s <filename> [IP address] [TCP port] [UDP port]\n\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	char address[] = "192.168.10.16";
	char *addr = (argc > n)? argv[n] : address; n++;
	int tcpPort = (argc > n)? atoi(argv[n]) : 24; n++;
	int udpPort = (argc > n)? atoi(argv[n]) : 4660; n++;

	try {
		VmeGbeConfig cfg(argv[1]);

		puts("\n************ Initialization ************");
		VmeGbeComm comm(addr, tcpPort);
		printf("%s(%d) connected\n", addr, tcpPort);

		printf("Download %s\n", argv[1]);
		if (!cfg.download(addr, udpPort, true))
			exit(EXIT_FAILURE);
		/*
		unsigned char eventFlg = 0x03;
		cfg.setEventEnabled(eventFlg);
		printf("Event Enabled: %.02x\n", eventFlg);
		*/

		unsigned int addr0 = 0x55555404;
		unsigned int valEnable = htonl(0x01000137);
		comm.write(addr0, sizeof(valEnable), cfg.getID(), VmeGbeComm::D32, (char *)&valEnable);
		printf("0x%.8x write at address<0x%.8x>\n", valEnable, addr0);
#if 1
		puts("\n************* Received Data ************\n");
		while (1) {
			std::vector<char> buf;
			struct VmeGbeComm::sitcp_vme_master_header header;
			comm.getData(header, buf);
			if (comm.bError(header))
				break;

			int priority = comm.getPriority(header);
			if (comm.bPolling(header) == false) {
				//printf("Interrupt%d   0x%.8x\n", priority, header.address);
				unsigned int val;
				if (header.address == 0x55555408 && buf.size() == sizeof(val)*2) {
					printf("Interrupt%d   ", priority);
					unsigned int *p = (unsigned int *)buf.data();
					val = ntohl(*p);
					printf("[0x%.8x] 0x%.8x   ", header.address, val);
					p++;
					val = ntohl(*p);
					printf("[0x%.8lx] 0x%.8x\n", header.address+sizeof(val), val);
				}
			}
			else {
				unsigned int val = 0;
				if (buf.size() == sizeof(val)) {
					unsigned int *p = (unsigned int *)buf.data();
					val = ntohl(*p);
					//printf("Polling(lv.%d) [0x%.8x] 0x%.8x\n", priority, header.address, val);
				}
				if (val == 0xAAAA5555)
					break;
			}
		}
#endif
		puts("\n************* Finalization *************");
		unsigned int valDisable = htonl(0x00000037);
		comm.write(addr0, sizeof(valDisable), cfg.getID(), VmeGbeComm::D32, (char *)&valDisable);
		printf("0x%.8x write at address<0x%.8x>\n", valDisable, addr0);

		cfg.clearEvent();
		puts("Event Flag Cleared (Event all mask)");

		comm.tcpClose();
		printf("%s(%d) connection closed\n", addr, tcpPort);
		puts("****************************************\n");
	}
	catch (std::string &estr) {
		puts(estr.c_str());
	}
	return EXIT_SUCCESS;
}
